@@ -1850,12 +1850,73 @@ class CreateTrackedFolderAction(flow.Action):
         self._files.touch()
 
 
+class FileInfos(flow.Object):
+
+    action = flow.Parent(2).ui(label="files")
+
+    path = flow.SessionParam("")
+    basename = flow.SessionParam("")
+    type = flow.SessionParam("")
+    extension = flow.SessionParam("")
+
+
+class DragFiles(flow.ConnectAction):
+
+    _file_map = flow.Parent()
+
+    def accept_label(self, objects, urls):
+        if not len(urls):
+            return None
+        
+        return "Drop %i file(s)/folder(s) here" % len(urls)
+
+    def run(self, objects, urls):
+        urls = list(map(lambda url: url.replace("file:///", ""), urls))
+        paths = ";".join(urls).lstrip(";").rstrip(";")
+        self._file_map.paths.set(paths)
+        self._file_map.touch()
+
+
+class FilesToAdd(flow.DynamicMap):
+
+    paths = flow.SessionParam("")
+    drag_files = flow.Child(DragFiles)
+
+    def mapped_names(self, page_num=0, page_size=None):
+        # Count number of non empty paths
+        path_count = sum(map(bool, self.paths.get().split(";")))
+        return ["file_%03i" % i for i in range(path_count)]
+    
+    @classmethod
+    def mapped_type(cls):
+        return FileInfos
+    
+    def _configure_child(self, child):
+        index = self.mapped_names().index(child.name())
+        paths = self.paths.get().split(";")
+        path = paths[index]
+        fsname, ext = os.path.splitext(path)
+        
+        child.path.set(path)
+        child.basename.set(os.path.basename(fsname))
+        child.type.set(
+            "file" if os.path.isfile(path) else "folder"
+        )
+        child.extension.set(ext[1:] if ext else "")
+
+
+class AddFilesFromExisting(flow.Action):
+
+    files = flow.Child(FilesToAdd)
+
+
 class FileSystemMap(flow.Map):
 
     create_file = flow.Child(CreateTrackedFileAction).injectable()
     create_folder = flow.Child(CreateTrackedFolderAction).injectable()
     create_untracked_folder = flow.Child(CreateFolderAction)
     create_untracked_file = flow.Child(CreateFileAction)
+    add_files_from_existing = flow.Child(AddFilesFromExisting)
 
     @classmethod
     def mapped_type(cls):
